import { sha256 } from "@noble/hashes/sha256";
import { encodeClock } from "./clock.js";
import { encodeSignedMessage, decodeSignedMessage } from "./schema.js";
import { KEY_LENGTH, decodeId } from "./ids.js";
export class SignedMessage {
    static decode(value) {
        const { signature, message } = decodeSignedMessage(value);
        return new SignedMessage(signature, message, value);
    }
    static encode(signature, message) {
        const value = encodeSignedMessage(signature, message);
        return new SignedMessage(signature, message, value);
    }
    constructor(signature, message, value) {
        this.signature = signature;
        this.message = message;
        this.value = value;
        this.key = getKey(message.clock, value);
        this.id = decodeId(this.key);
    }
}
function getKey(clock, value) {
    const hash = sha256(value);
    const key = new Uint8Array(KEY_LENGTH);
    const encodingLength = encodeClock(key, clock);
    key.set(hash.subarray(0, KEY_LENGTH - encodingLength), encodingLength);
    return key;
}
